@@ -124,43 +124,39 @@ file_node_data_free (FileNodeData *data,
 	g_slice_free (FileNodeData, data);
 }
 
-static FileNodeData *
+static GNode *
 file_node_data_new (TrackerFileSystem *file_system,
                     GFile             *file,
                     GFileType          file_type,
-                    GNode             *node)
+                    gchar             *uri_prefix)
 {
 	FileNodeData *data;
-	NodeLookupData lookup_data;
-	GArray *node_data;
+	NodeLookupData *lookup_data;
 
 	data = g_slice_new0 (FileNodeData);
 	data->file = g_object_ref (file);
 	data->file_type = file_type;
+	data->uri_prefix = uri_prefix;
 	data->properties = g_array_new (FALSE, TRUE, sizeof (FileNodeProperty));
 
-	/* We use weak refs to keep track of files */
-	g_object_weak_ref (G_OBJECT (data->file), file_weak_ref_notify, node);
+	lookup_data = g_object_get_qdata (G_OBJECT (data->file),
+	                                  quark_file_node);
 
-	node_data = g_object_get_qdata (G_OBJECT (data->file),
-	                                quark_file_node);
-
-	if (!node_data) {
-		node_data = g_array_new (FALSE, FALSE, sizeof (NodeLookupData));
+	if (!lookup_data) {
+		lookup_data = g_new0 (NodeLookupData, 1);
 		g_object_set_qdata_full (G_OBJECT (data->file),
 		                         quark_file_node,
-		                         node_data,
-		                         (GDestroyNotify) g_array_unref);
+		                         lookup_data, g_free);
 	}
 
-	lookup_data.file_system = file_system;
-	lookup_data.node = node;
-	g_array_append_val (node_data, lookup_data);
+	lookup_data->file_system = file_system;
+	lookup_data->node = g_node_new (data);
 
-	g_assert (node->data == NULL);
-	node->data = data;
+	/* We use weak refs to keep track of files */
+	g_object_weak_ref (G_OBJECT (data->file),
+	                   file_weak_ref_notify, lookup_data->node);
 
-	return data;
+	return lookup_data->node;
 }
 
 static FileNodeData *
@@ -468,10 +464,6 @@ reparent_child_nodes_to_parent (GNode *node)
 	FileNodeData *node_data;
 	GNode *child, *parent;
 
-	if (!node->parent) {
-		return;
-	}
-
 	parent = node->parent;
 	node_data = node->data;
 	child = g_node_first_child (node);
@@ -493,8 +485,12 @@ reparent_child_nodes_to_parent (GNode *node)
 		data->uri_prefix = uri_prefix;
 
 		g_node_unlink (cur);
-		g_node_prepend (parent, cur);
+
+		if (parent)
+			g_node_prepend (parent, cur);
 	}
+
+	g_assert (!g_node_first_child (node));
 }
 
 static void
@@ -522,31 +518,15 @@ file_system_get_node (TrackerFileSystem *file_system,
                       GFile             *file)
 {
 	TrackerFileSystemPrivate *priv;
-	GArray *node_data;
-	GNode *node = NULL;
+	NodeLookupData *lookup_data;
 
-	node_data = g_object_get_qdata (G_OBJECT (file), quark_file_node);
+	lookup_data = g_object_get_qdata (G_OBJECT (file), quark_file_node);
 
-	if (node_data) {
-		NodeLookupData *cur;
-		guint i;
+	if (lookup_data && lookup_data->file_system == file_system)
+		return lookup_data->node;
 
-		for (i = 0; i < node_data->len; i++) {
-			cur = &g_array_index (node_data, NodeLookupData, i);
-
-			if (cur->file_system == file_system) {
-				node = cur->node;
-			}
-		}
-	}
-
-	if (!node) {
-		priv = file_system->priv;
-		node = file_tree_lookup (priv->file_tree, file,
-		                         NULL, NULL);
-	}
-
-	return node;
+	priv = file_system->priv;
+	return file_tree_lookup (priv->file_tree, file, NULL, NULL);
 }
 
 GFile *
@@ -556,26 +536,45 @@ tracker_file_system_get_file (TrackerFileSystem *file_system,
                               GFile             *parent)
 {
 	TrackerFileSystemPrivate *priv;
+	NodeLookupData *lookup_data;
 	FileNodeData *data;
-	GNode *node, *parent_node;
+	GNode *node, *parent_node, *lookup_node = NULL;
 	gchar *uri_prefix = NULL;
+	GFile *copy = NULL;
 
 	g_return_val_if_fail (G_IS_FILE (file), NULL);
 	g_return_val_if_fail (TRACKER_IS_FILE_SYSTEM (file_system), NULL);
 
 	priv = file_system->priv;
 	node = NULL;
-	parent_node = NULL;
+	lookup_data = g_object_get_qdata (G_OBJECT (file), quark_file_node);
+
+	/* If file is interned somewhere else, get a separate copy of the
+	 * file for this filesystem.
+	 */
+	if (lookup_data && lookup_data->file_system != file_system) {
+		gchar *uri = g_file_get_uri (file);
+		copy = g_file_new_for_uri (uri);
+		g_free (uri);
+		file = copy;
+	} else if (lookup_data) {
+		/* Short circuit path, file is already interned */
+		return file;
+	}
 
 	if (parent) {
 		parent_node = file_system_get_node (file_system, parent);
-		node = file_tree_lookup (parent_node, file,
-		                         NULL, &uri_prefix);
-	} else {
-		node = file_tree_lookup (priv->file_tree, file,
-		                         &parent_node, &uri_prefix);
+
+		if (parent_node)
+			lookup_node = parent_node;
 	}
 
+	if (!lookup_node) {
+		lookup_node = priv->file_tree;
+	}
+
+	node = file_tree_lookup (lookup_node, file, &parent_node, &uri_prefix);
+
 	if (!node) {
 		if (!parent_node) {
 			gchar *uri;
@@ -585,17 +584,17 @@ tracker_file_system_get_file (TrackerFileSystem *file_system,
 			g_warning ("NOTE: URI theme may be outside scheme expected, for example, expecting 'file://' when given 'http://' prefix.");
 			g_free (uri);
 
+			g_clear_object (&copy);
+
 			return NULL;
 		}
 
-		node = g_node_new (NULL);
-
 		/* Parent was found, add file as child */
-		data = file_node_data_new (file_system, file,
-		                           file_type, node);
-		data->uri_prefix = uri_prefix;
+		node = file_node_data_new (file_system, file,
+		                           file_type, uri_prefix);
 
 		g_node_append (parent_node, node);
+		data = node->data;
 	} else {
 		data = node->data;
 		g_free (uri_prefix);
@@ -606,6 +605,8 @@ tracker_file_system_get_file (TrackerFileSystem *file_system,
 		}
 	}
 
+	g_clear_object (&copy);
+
 	return data->file;
 }
 
